/*******************************************************************************
 *
 * File Name: LRRCcombat.cpp
 * Author: Richard Baxter
 * Project: Lego Rules CG Rounds Checker
 * Project Version: 3e2a 16-Feb-07
 * Project First Internal Release: 1aXx 18-Sept-05 (C)
 * Project Second Internal Release: 2aXx 02-April-06 (convert to C++)
 * Project Third Internal Release: 2b7d 26-Sept-06 (added sprites)
 * Project Fourth Internal Release: 2c4a 25-Feb-07 [rotation of models supported, copyCombatRelevantInheritedImmediateParentUnitDetailsIntoChildObject() no longer used]
 * Project Fifth Internal Release: 2d11a 3-June-07
 *
 *******************************************************************************/


#include "LRRCcombat.h"
#include "LDparser.h"
#include "XMLrulesClass.h"


static double CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY;
static double RANGE_MODIFIER_BASED_ON_INCREASED_HEIGHT;



void fillInCombatExternVariables()
{
	RulesClass * currentReferenceRulesClass = LRRCrulesMiscellaneous;

	while(currentReferenceRulesClass->next != NULL)
	{
		if(currentReferenceRulesClass->name == CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY_NAME)
		{
			CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY = currentReferenceRulesClass->fractionalValue;
		}
		else if(currentReferenceRulesClass->name == RANGE_MODIFIER_BASED_ON_INCREASED_HEIGHT_NAME)
		{
			RANGE_MODIFIER_BASED_ON_INCREASED_HEIGHT = currentReferenceRulesClass->fractionalValue;
		}
		else
		{

		}

		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}
}


























































































/*Basic: This method does not work with embedded units, and assumes the unit does not contain submodels relevant to its class*/
bool calculateUnitClassBasic(char* unitFileName)
{
	//reinitalise ModelDetails specific values if necessary
	Reference * initialReferenceInUnitFile = new Reference();
	Reference * topLevelReferenceInFile = new Reference(true);
	ModelDetails * u = topLevelReferenceInFile->subModelDetails;

	if(!parseFile(unitFileName, initialReferenceInUnitFile, topLevelReferenceInFile))
	{//file does not exist
		return false;
	}
	else
	{
		performFinalUnitClassCalculations(u);
		if(!performUnitOrdinatesCheck(u))
		{
			return false;
		}
		cout << "Defence Level = " << invertLevel(u->defenceTotal) << endl;
		cout << "Attack Level = " << invertLevel(u->attackTotal) << endl;
		return true;
	}
}

//new
bool calculateUnitClassNormal(char* unitFileName)
{
	//declare initial scene references
	Reference * initialReferenceInUnit1 = new Reference();
	Reference * topLevelReferenceInUnit1 = new Reference(true);

	//parse the unit files and build Reference linked lists
	parseFile(unitFileName, initialReferenceInUnit1, topLevelReferenceInUnit1);

	//initialise unit1/unit2 references
	topLevelReferenceInUnit1->colour = int(DEFAULT_COLOUR_OF_SUB_MODEL_CURRENTLY_BEING_SEARCHED_FOR);	// this should be modified appropriatly in the future when multiple references to the same ModelDetails can be included in a scene file (of different colours)
	topLevelReferenceInUnit1->name = unitFileName;

	//determing the current position of Unit 1
	searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(topLevelReferenceInUnit1, initialReferenceInUnit1, NULL, true);

			//the third parameter is not used, as the forth parameter is set true.
	performFinalUnitClassCalculations(topLevelReferenceInUnit1->subModelDetails);
	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit1->subModelDetails))
	{
		return false;
	}

	cout << "Defence Level = " << invertLevel(topLevelReferenceInUnit1->subModelDetails->defenceTotal) << endl;
	cout << "Attack Level = " << invertLevel(topLevelReferenceInUnit1->subModelDetails->attackTotal) << endl;

	return true;
}

























































































































































/*
//preconditions: Assumes particular ModelDetails files exist and are available. Assumes ModelDetails files are fully self contained ModelDetails files - ie each ModelDetails file must contain all information including that about relevant horses etc
//
//postconditions: returns 1 if both strike, 2 if unit1 wins, 3 if unit2 wins, 4 if neither strike, 5 if neither can strike
	//							6 if unit1 wins and only Unit 1 can strike, 7 if unit1 does not strike, and only Unit 1 can strike, 8 if  3 if unit2 wins, 4 if neither strike
	//							8 if unit2 wins and only Unit 2 can strike, 9 if unit2 does not strike, and only Unit 2 can strike.
	//	0/10 if error
*/
int performCloseCombatBasic(char* unit1FileName, char* unit2FileName, bool unit1performsCloseCombatAttack, bool unit2performsCloseCombatAttack, bool unit1TakesTheInitative, bool unit2TakesTheInitative)
{
	//declare variables references
	Reference * initialReferenceInUnit1 = new Reference();
	Reference * initialReferenceInUnit2 = new Reference();
	Reference * topLevelReferenceInUnit1 = new Reference(true);
	Reference * topLevelReferenceInUnit2 = new Reference(true);

	//parse the scene files and build Reference linked lists
	if(!parseFile(unit1FileName, initialReferenceInUnit1, topLevelReferenceInUnit1))
	{//file does not exist
		cout << "The unit unit1FileName=" << unit1FileName << " does not exist in the directory" << endl;
		return false;
	}

	if(!parseFile(unit2FileName, initialReferenceInUnit2, topLevelReferenceInUnit2))
	{//file does not exist
		cout << "The unit unit2FileName=" << unit2FileName << " does not exist in the directory" << endl;
		return false;
	}

	performFinalUnitClassCalculations(topLevelReferenceInUnit1->subModelDetails);
	performFinalUnitClassCalculations(topLevelReferenceInUnit2->subModelDetails);

	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit2->subModelDetails))
	{
		return false;
	}
	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit2->subModelDetails))
	{
		return false;
	}

	return performCloseCombat(topLevelReferenceInUnit1->subModelDetails, topLevelReferenceInUnit2->subModelDetails, unit1performsCloseCombatAttack, unit2performsCloseCombatAttack, unit1TakesTheInitative, unit2TakesTheInitative);
}



//returns 1 if both strike, 2 if unit1 wins, 3 if unit2 wins, 4 if neither strike. Return 5 if close combat could not occur (neither could strike).
	//							6 if unit1 wins and only Unit 1 can strike, 7 if unit1 does not strike, and only Unit 1 can strike, 8 if  3 if unit2 wins, 4 if neither strike
	//							8 if unit2 wins and only Unit 2 can strike, 9 if unit2 does not strike, and only Unit 2 can strike.
	//							0/10 if error in finding required files.
int performCloseCombatNormal(char* unit1FileName, char* unit2FileName, bool unit1performsCloseCombatAttack, bool unit2performsCloseCombatAttack)
{
	//declare variables references
	Reference * initialReferenceInUnit1 = new Reference();
	Reference * initialReferenceInUnit2 = new Reference();
	Reference * topLevelReferenceInUnit1 = new Reference(true);
	Reference * topLevelReferenceInUnit2 = new Reference(true);

	//parse the unit files and build Reference linked lists
	if(!parseFile(unit1FileName, initialReferenceInUnit1, topLevelReferenceInUnit1))
	{//file does not exist
		cout << "The unit unit1FileName=" << unit1FileName << " does not exist in the directory" << endl;
		return false;
	}

	//parse the unit files and build Reference linked lists
	if(!parseFile(unit2FileName, initialReferenceInUnit2, topLevelReferenceInUnit2))
	{//file does not exist
		cout << "The unit unit2FileName=" << unit2FileName << " does not exist in the directory" << endl;
		return false;
	}


	//initialise unit1/unit2 references
	topLevelReferenceInUnit1->colour = int(DEFAULT_COLOUR_OF_SUB_MODEL_CURRENTLY_BEING_SEARCHED_FOR);	// this should be modified appropriatly in the future when multiple references to the same ModelDetails can be included in a scene file (of different colours)
	topLevelReferenceInUnit1->name = unit1FileName;
	topLevelReferenceInUnit2->colour = int(DEFAULT_COLOUR_OF_SUB_MODEL_CURRENTLY_BEING_SEARCHED_FOR);	// this should be modified appropriatly in the future when multiple references to the same ModelDetails can be included in a scene file (of different colours)
	topLevelReferenceInUnit2->name = unit2FileName;


	//determing the current position of Unit 1
	searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(topLevelReferenceInUnit1, initialReferenceInUnit1, NULL, true);

	performFinalUnitClassCalculations(topLevelReferenceInUnit1->subModelDetails);

	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit1->subModelDetails))
	{
		return false;
	}

	//determing the current position of Unit 2
	searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(topLevelReferenceInUnit2, initialReferenceInUnit2, NULL, true);

	performFinalUnitClassCalculations(topLevelReferenceInUnit2->subModelDetails);
	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit2->subModelDetails))
	{
		return false;
	}

	bool unit1TakesTheInitative = true;
	bool unit2TakesTheInitative = true;


	return performCloseCombat(topLevelReferenceInUnit1->subModelDetails, topLevelReferenceInUnit2->subModelDetails,  unit1performsCloseCombatAttack,  unit2performsCloseCombatAttack, unit1TakesTheInitative, unit2TakesTheInitative);
			//in the future, before performCloseCombat is invoked, must check in current scene file, and previous scene file whether both units could strike during close combat
			//Eg the knight charge first strike policy will prevent at some times some units from being able to strike first

}

//preconditions: units are assumed to have moved in their previous round (ie they both take the initative)
//returns 1 if both strike, 2 if unit1 wins, 3 if unit2 wins, 4 if neither strike. Return 5 if close combat could not occur (neither could strike).
	//							6 if unit1 wins and only Unit 1 can strike, 7 if unit1 does not strike, and only Unit 1 can strike, 8 if  3 if unit2 wins, 4 if neither strike
	//							8 if unit2 wins and only Unit 2 can strike, 9 if unit2 does not strike, and only Unit 2 can strike.
	//							0/10 if error in finding required files.
int performCloseCombatWithSceneFile(char* unit1FileName, char* unit2FileName, int unit1ID, int unit2ID, bool unit1intendsToPerformCloseCombatAttack, bool unit2intendsToPerformCloseCombatAttack, char* thisPhaseStartSceneFileName)
{

	//declare initial scene references
	Reference * initialReferenceInThisPhaseStartScene = new Reference();
	Reference * topLevelReferenceInThisPhaseStartScene = new Reference(true);	//the modelDetails information in this object is not required or meaningful, but needs to be passed into the parseFile/parseReferenceList recursive function

	//parse the scene files and build Reference linked lists
	if(!parseFile(thisPhaseStartSceneFileName, initialReferenceInThisPhaseStartScene, topLevelReferenceInThisPhaseStartScene))
	{//file does not exist
		cout << "The file thisPhaseStartSceneFileName " << thisPhaseStartSceneFileName << " does not exist in the directory" << endl;
		return false;
	}
	//cout << "DEBUG: b1" << endl;

	return performCloseCombatWithScene(unit1FileName, unit2FileName, unit1ID, unit2ID, unit1intendsToPerformCloseCombatAttack, unit2intendsToPerformCloseCombatAttack, initialReferenceInThisPhaseStartScene);

}

int performCloseCombatWithScene(char* unit1FileName, char* unit2FileName, int unit1ID, int unit2ID, bool unit1intendsToPerformCloseCombatAttack, bool unit2intendsToPerformCloseCombatAttack, Reference * initialReferenceInThisPhaseStartScene)
{

	//initialise unit1/unit2 references		[NB in the future an appropriate constructor should be created for the Reference class that accepts a name, and automatically creates a subModelDetails ModelDetails*/
	Reference * unit1ReferenceInThisPhaseStartSceneFile = new Reference(unit1FileName, unit1ID, true);
	Reference * unit2ReferenceInThisPhaseStartSceneFile = new Reference(unit2FileName, unit2ID, true);

	//determing the current position of Unit 1
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit1ReferenceInThisPhaseStartSceneFile, initialReferenceInThisPhaseStartScene, NULL, false))
	{
		cout << "The unit unit1FileName=" << unit1FileName << " unit1ID=" << unit1ID << " does not exist in the given scene" << endl;
		//cout << "DEBUG: b3" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails))
	{
		return false;
	}

	//determing the current position of Unit 2
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit2ReferenceInThisPhaseStartSceneFile, initialReferenceInThisPhaseStartScene, NULL, false))
	{
		cout << "The unit unit2FileName=" << unit2FileName << " unit2ID=" << unit2ID << " does not exist in the given scene" << endl;
		//cout << "DEBUG: b7" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit2ReferenceInThisPhaseStartSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit2ReferenceInThisPhaseStartSceneFile->subModelDetails))
	{
		return false;
	}


	bool unit1TakesTheInitative = true;
	bool unit2TakesTheInitative = true;

	//check whether current units are within Close Combat Range
	double distanceBetweenTheTwoUnits = calculateTheDistanceBetweenTwoPoints(unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.x, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.x, unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y, unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.z, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.z);
	if(distanceBetweenTheTwoUnits <= CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY)
	{
		//perform close combat
		return performCloseCombat(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails, unit2ReferenceInThisPhaseStartSceneFile->subModelDetails,  unit1intendsToPerformCloseCombatAttack,  unit2intendsToPerformCloseCombatAttack, unit1TakesTheInitative, unit2TakesTheInitative);
			//in the future, before performCloseCombat is invoked, must check in current scene file, and previous scene file whether both units could strike during close combat
			//Eg the knight charge first strike policy will prevent at some times some units from being able to strike first
	}
	else
	{
		cout << "\nDEBUG: Units are out of Range. Distance between units = " << distanceBetweenTheTwoUnits << endl;
		cout << "DEBUG: Maximum Distance = " << CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY << endl;
		return ATTACK_NEITHER_UNIT_CAN_STRIKE;
	}
}




//postconditions: the validity of the movement of the units(') between scene files is not checked - this should be done in the round's movement checker routine
//returns 1 if both strike, 2 if unit1 wins, 3 if unit2 wins, 4 if neither strike. Return 5 if close combat could not occur (neither could strike).
	//							6 if unit1 wins and only Unit 1 can strike, 7 if unit1 does not strike, and only Unit 1 can strike, 8 if  3 if unit2 wins, 4 if neither strike
	//							8 if unit2 wins and only Unit 2 can strike, 9 if unit2 does not strike, and only Unit 2 can strike.
	//							0/10 if error in finding required files.
int performCloseCombatWithConsecutiveSceneFiles(char* unit1FileName, char* unit2FileName, int unit1ID, int unit2ID, bool unit1intendsToPerformCloseCombatAttack, bool unit2intendsToPerformCloseCombatAttack, char* thisPhaseStartSceneFileName, char* preMovementPhaseSceneFileName)
{

	//declare initial scene references
	Reference * initialReferenceInThisPhaseStartScene = new Reference();
	Reference * initialReferenceInPreMovementPhaseScene = new Reference();
	Reference * topLevelReferenceInThisPhaseStartScene = new Reference(true);	//the smodelDetails information in this object is not required or meaningful, but needs to be passed into the parseFile/parseReferenceList recursive function
	Reference * topLevelReferenceInPreMovementPhaseScene = new Reference(true);	//the modelDetails information in this object is not required or meaningful, but needs to be passed into the parseFile/parseReferenceList recursive function

	//parse the scene files and build Reference linked lists
	if(!parseFile(thisPhaseStartSceneFileName, initialReferenceInThisPhaseStartScene, topLevelReferenceInThisPhaseStartScene))
	{//file does not exist
		cout << "The file: " << thisPhaseStartSceneFileName << " does not exist in the directory" << endl;
		return false;
	}
	//cout << "DEBUG: b1" << endl;

	if(!parseFile(preMovementPhaseSceneFileName, initialReferenceInPreMovementPhaseScene, topLevelReferenceInPreMovementPhaseScene))
	{//file does not exist
		cout << "The file: " << preMovementPhaseSceneFileName << " does not exist in the directory" << endl;
		return false;
	}
	//cout << "DEBUG: b2" << endl;


	return performCloseCombatWithConsecutiveScenes(unit1FileName, unit2FileName, unit1ID, unit2ID, unit1intendsToPerformCloseCombatAttack, unit2intendsToPerformCloseCombatAttack, initialReferenceInThisPhaseStartScene, initialReferenceInPreMovementPhaseScene);

}



int performCloseCombatWithConsecutiveScenes(char* unit1FileName, char* unit2FileName, int unit1ID, int unit2ID, bool unit1intendsToPerformCloseCombatAttack, bool unit2intendsToPerformCloseCombatAttack, Reference * initialReferenceInThisPhaseStartScene, Reference * initialReferenceInPreMovementPhaseScene)
{
	//initialise unit1/unit2 references		[NB in the future an appropriate constructor should be created for the Reference class that accepts a name, and automatically creates a subModelDetails ModelDetails*/

	Reference * unit1ReferenceInThisPhaseStartSceneFile = new Reference(unit1FileName, unit1ID, true);
	Reference * unit2ReferenceInThisPhaseStartSceneFile = new Reference(unit2FileName, unit2ID, true);
	Reference * unit1ReferenceInPreMovementPhaseSceneFile = new Reference(unit1FileName, unit1ID, true);
	Reference * unit2ReferenceInPreMovementPhaseSceneFile = new Reference(unit2FileName, unit2ID, true);

	//determing the current position of Unit 1
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit1ReferenceInThisPhaseStartSceneFile, initialReferenceInThisPhaseStartScene, NULL, false))
	{
		cout << "The unit unit1FileName=" << unit1FileName << " unit1ID=" << unit1ID << " does not exist in the given scene" << endl;
		cout << "unit1ReferenceInThisPhaseStartSceneFile->colour = " << unit1ReferenceInThisPhaseStartSceneFile->colour << endl;
		cout << "unit1ReferenceInThisPhaseStartSceneFile->name = " << unit1ReferenceInThisPhaseStartSceneFile->name << endl;
		cout << "DEBUG: b3" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails))
	{
		return false;
	}

	//determing the previous  position of Unit 1
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit1ReferenceInPreMovementPhaseSceneFile, initialReferenceInPreMovementPhaseScene, NULL, false))
	{
		cout << "The unit unit1FileName=" << unit1FileName << " unit1ID=" << unit1ID << " does not exist in the given" << endl;
		cout << "DEBUG: b5" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit1ReferenceInPreMovementPhaseSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit1ReferenceInPreMovementPhaseSceneFile->subModelDetails))
	{
		return false;
	}

	//determing the current position of Unit 2
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit2ReferenceInThisPhaseStartSceneFile, initialReferenceInThisPhaseStartScene, NULL, false))
	{
		cout << "The unit unit2FileName=" << unit1FileName << " unit2ID=" << unit2ID << " does not exist in the given scene" << endl;
		cout << "DEBUG: b7" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit2ReferenceInThisPhaseStartSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit2ReferenceInThisPhaseStartSceneFile->subModelDetails))
	{
		return false;
	}

	//determing the previous position of Unit 2
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit2ReferenceInPreMovementPhaseSceneFile, initialReferenceInPreMovementPhaseScene, NULL, false))
	{
		cout << "The unit unit2FileName=" << unit1FileName << " unit2ID=" << unit2ID << " does not exist in the given scene" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit2ReferenceInPreMovementPhaseSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit2ReferenceInPreMovementPhaseSceneFile->subModelDetails))
	{
		return false;
	}

	//check whether a ModelDetails has advanced into combat, and therefore takes the initative to strike first.

	bool unit1HasMovedInPreviousRound = false;
	bool unit2HasMovedInPreviousRound = false;

		//26-3-06 'absolute' changed to 'relative' reasoning: /*"A movement turn does not only count for the individual " ...change rules to accomodate long distance attack if an absolute movement has occured but a relative has not for any given unit.*/
	double distanceMovedByUnit1DuringRound = calculateTheDistanceBetweenTwoPoints(unit1ReferenceInThisPhaseStartSceneFile->relativePosition.x, unit1ReferenceInPreMovementPhaseSceneFile->relativePosition.x, unit1ReferenceInThisPhaseStartSceneFile->relativePosition.y, unit1ReferenceInPreMovementPhaseSceneFile->relativePosition.y, unit1ReferenceInThisPhaseStartSceneFile->relativePosition.z, unit1ReferenceInPreMovementPhaseSceneFile->relativePosition.z);
	double distanceMovedByUnit2DuringRound = calculateTheDistanceBetweenTwoPoints(unit2ReferenceInThisPhaseStartSceneFile->relativePosition.x, unit2ReferenceInPreMovementPhaseSceneFile->relativePosition.x, unit2ReferenceInThisPhaseStartSceneFile->relativePosition.y, unit2ReferenceInPreMovementPhaseSceneFile->relativePosition.y, unit2ReferenceInThisPhaseStartSceneFile->relativePosition.z, unit2ReferenceInPreMovementPhaseSceneFile->relativePosition.z);

	if(int(distanceMovedByUnit1DuringRound) != 0)
	{
		unit1HasMovedInPreviousRound = true;
	}
	if(int(distanceMovedByUnit2DuringRound) != 0)
	{
		unit2HasMovedInPreviousRound = true;
	}

	bool unit1TakesTheInitative = false;
	bool unit2TakesTheInitative = false;

	if(unit1HasMovedInPreviousRound)		/*before 26-3-06: if(unit1HasMovedInPreviousRound && !unit2HasMovedInPreviousRound)*/
	{
		unit1TakesTheInitative = true;
	}
	else if(unit2HasMovedInPreviousRound)	/*before 26-3-06: else if(unit2HasMovedInPreviousRound && !unit1HasMovedInPreviousRound)*/
	{
		unit2TakesTheInitative = true;
	}

	//cout << "DEBUG: b5" << endl;

	//check whether current units are within Close Combat Range
	double distanceBetweenTheTwoUnits = calculateTheDistanceBetweenTwoPoints(unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.x, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.x, unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y, unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.z, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.z);
	if(distanceBetweenTheTwoUnits <= CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY)
	{
		//perform close combat
		return performCloseCombat(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails, unit2ReferenceInThisPhaseStartSceneFile->subModelDetails,  unit1intendsToPerformCloseCombatAttack,  unit2intendsToPerformCloseCombatAttack, unit1TakesTheInitative, unit2TakesTheInitative);
			//in the future, before performCloseCombat is invoked, must check in current scene file, and previous scene file whether both units could strike during close combat
			//Eg the knight charge first strike policy will prevent at some times some units from being able to strike first
	}
	else
	{
		cout << "\nDEBUG: Units are out of Range. Distance between units = " << distanceBetweenTheTwoUnits << endl;
		cout << "DEBUG: Maximum Distance = " << CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY << endl;
		return ATTACK_NEITHER_UNIT_CAN_STRIKE;
	}
}






/*
//preconditions: Assumes particular ModelDetails objects (unit1 and unit2) are fully self contained - ie they contain all information including that about horses etc
//		(even though a knights respective horse may have been found in a higher level file during initial parse)
//
//postconditions: returns 1 if both strike, 2 if unit1 wins, 3 if unit2 wins, 4 if neither strike, 5 if neither can strike
	//							6 if unit1 wins and only Unit 1 can strike, 7 if unit1 does not strike, and only Unit 1 can strike, 8 if  3 if unit2 wins, 4 if neither strike
	//							8 if unit2 wins and only Unit 2 can strike, 9 if unit2 does not strike, and only Unit 2 can strike.
	//	0/10 if error
*/
int performCloseCombat(ModelDetails * unit1, ModelDetails * unit2, bool unit1performsCloseCombatAttack, bool unit2performsCloseCombatAttack, bool unit1TakesTheInitative, bool unit2TakesTheInitative)
{
	int result;

	int unit1Attack;
	int unit1Defence;
	unit1Defence = unit1->defenceTotal;
	unit1Attack = calculateCloseCombatAttackBonus(unit1, (bool)(unit2->numHorse), unit1TakesTheInitative);

	int unit2Attack;
	int unit2Defence;
	unit2Defence = unit2->defenceTotal;
	unit2Attack = calculateCloseCombatAttackBonus(unit2, (bool)(unit1->numHorse), unit2TakesTheInitative);

#ifdef GAME_OUTPUT_COMBAT_ENTRY_STATISTICS
	cout << "\nUnit 1 Combat Entry:" << endl;
	cout << "Defence Level = " << invertLevel(unit1->defenceTotal) << endl;
	cout << "Close Combat Attack Level = " << unit1Attack << endl;

	cout << "\nUnit 2 Combat Entry:" << endl;
	cout << "Defence Level = " << invertLevel(unit2->defenceTotal) << endl;
	cout << "Close Combat Attack Level = " << unit2Attack << endl;
#endif


	/*ADD_IF_USING_DIFFERENT_VERSION_OF_RULES_WHERE_CLOSE_COMBAT_FINISHES_IN_ONE_ROUND
	bool stillFighting = true;
	while(stillFighting)
	{
	*/

	bool unit1Strikes = false;
	bool unit2Strikes = false;

#ifdef USE_UNACCEPTED_2006_RULES
	if((unit1TakesTheInitative && unit1->numHorse && !unit2->numHorse && !unit2->numSpear) ||
		(unit1TakesTheInitative && unit1->numHorse && (unit1->numLance || unit1->numSpear) && unit2->numHorse && !(unit2->numLance || unit2->numSpear)))
	{
		//cout << "(unit1TakesTheInitative && unit1->numHorse && !unit2->numHorse) && !unit2->numSpear) \n || \n (unit1TakesTheInitative && unit1->numHorse && (unit1->numLance || unit1->numSpear) && unit2->numHorse && !(unit2->numLance || unit2->numSpear))" << endl;
		//unit 1 takes the advantage - only if it fails to strike will unit 2 have a chance to strike
		if(unit1performsCloseCombatAttack)
		{
			int diceThrown1 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit1Attack)*DICE_MODIFIER+1.0);	 //+1 for 0.0001->11.9999 ... 1.0001->12.9999 ... 1->12 (round down)
			int diceThrown2 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit2Defence)*DICE_MODIFIER+1.0);

			cout << "unit 1 throws: " << diceThrown1 << " in attack using d" << (unit1Attack)*DICE_MODIFIER <<endl;
			cout << "unit 2 throws: " << diceThrown2 << " in defence using d" << (unit2Defence)*DICE_MODIFIER << endl;

			if(diceThrown1 > diceThrown2)
			{
				unit1Strikes = true;
			}
		}
		if(!unit1Strikes)
		{
			if(unit2performsCloseCombatAttack)
			{
				int diceThrown3 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit2Attack)*DICE_MODIFIER+1.0);
				int diceThrown4 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit1Defence)*DICE_MODIFIER+1.0);
				cout << "unit 2 throws: " << diceThrown3 << " in attack using d" << (unit2Attack)*DICE_MODIFIER << endl;
				cout << "unit 1 throws: " << diceThrown4 << " in defence using d" << (unit1Defence)*DICE_MODIFIER << endl;

				if(diceThrown3 > diceThrown4)
				{
					unit2Strikes = true;
				}
			}
		}
	}
	else if((unit2TakesTheInitative && unit2->numHorse && !unit1->numHorse && !unit1->numSpear) ||
		(unit2TakesTheInitative && unit2->numHorse && (unit2->numLance || unit2->numSpear) && unit1->numHorse && !(unit1->numLance || unit1->numSpear)))
	{
		//unit 2 takes the advantage - only if it fails to strike will unit 1 have a chance to strike
		if(unit2performsCloseCombatAttack)
		{
			int diceThrown3 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit2Attack)*DICE_MODIFIER+1.0);
			int diceThrown4 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit1Defence)*DICE_MODIFIER+1.0);
			cout << "unit 2 throws: " << diceThrown3 << " in attack using d" << (unit2Attack)*DICE_MODIFIER << endl;
			cout << "unit 1 throws: " << diceThrown4 << " in defence using d" << (unit1Defence)*DICE_MODIFIER << endl;

			if(diceThrown3 > diceThrown4)
			{
				unit2Strikes = true;
			}
		}

		if(!unit2Strikes)
		{
			if(unit1performsCloseCombatAttack)
			{
				int diceThrown1 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit1Attack)*DICE_MODIFIER+1.0);	 //+1 for 0.0001->11.9999 ... 1.0001->12.9999 ... 1->12 (round down)
				int diceThrown2 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit2Defence)*DICE_MODIFIER+1.0);

				cout << "unit 1 throws: " << diceThrown1 << " in attack using d" << (unit1Attack)*DICE_MODIFIER <<endl;
				cout << "unit 2 throws: " << diceThrown2 << " in defence using d" << (unit2Defence)*DICE_MODIFIER << endl;

				if(diceThrown1 > diceThrown2)
				{
					unit1Strikes = true;
				}
			}
		}
	}
	else
	{//(unit1TakesTheInitative && unit2TakesTheInitative) || (!unit2TakesTheInitative && !unit2TakesTheInitative)
#endif
		//allow both units to strike equally
		if(unit1performsCloseCombatAttack)
		{
			int diceThrown1 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit1Attack)*DICE_MODIFIER+1.0);	//+1 for 0.0001->11.9999 ... 1.0001->12.9999 ... 1->12 (round down)
			int diceThrown2 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit2Defence)*DICE_MODIFIER+1.0);

			cout << "unit 1 throws: " << diceThrown1 << " in attack using d" << (unit1Attack)*DICE_MODIFIER <<endl;
			cout << "unit 2 throws: " << diceThrown2 << " in defence using d" << (unit2Defence)*DICE_MODIFIER << endl;

			if(diceThrown1 > diceThrown2)
			{
				unit1Strikes = true;
			}
		}

		if(unit2performsCloseCombatAttack)
		{
			int diceThrown3 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit2Attack)*DICE_MODIFIER+1.0);
			int diceThrown4 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit1Defence)*DICE_MODIFIER+1.0);
			cout << "unit 2 throws: " << diceThrown3 << " in attack using d" << (unit2Attack)*DICE_MODIFIER << endl;
			cout << "unit 1 throws: " << diceThrown4 << " in defence using d" << (unit1Defence)*DICE_MODIFIER << endl;

			if(diceThrown3 > diceThrown4)
			{
				unit2Strikes = true;
			}
		}
#ifdef USE_UNACCEPTED_2006_RULES
	}
#endif

	if(unit1Strikes && unit2Strikes)
	{
		//nothing happens, no blows were fatal
		cout << "Both units strike. No strikes were fatal. [No units should be removed from scene file]." << endl;
		return ATTACK_BOTH_UNITS_STRIKE_BOTH_CAN_STRIKE;
	}
	else if(unit1Strikes && !unit2Strikes)
	{
		cout << "Unit 1 wins. [Unit 2 should be removed from the scene file]." << endl;
		if(unit2performsCloseCombatAttack)
		{
			result = ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE;
		}
		else
		{
			result = ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE;
		}
	}
	else if(!unit1Strikes && unit2Strikes)
	{
		cout << "Unit 2 wins. [Unit 1 should be removed from the scene file]." << endl;
		if(unit1performsCloseCombatAttack)
		{
			result = ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE;
		}
		else
		{
			result = ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE;
		}

	}
	else if(!unit1Strikes && !unit2Strikes)
	{
		cout << "The unit(s) intending to strike did not. [No units should be removed from scene file]." << endl;
		if(unit1performsCloseCombatAttack && unit2performsCloseCombatAttack)
		{
			result = ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE;
		}
		else if(!unit2performsCloseCombatAttack)
		{
			result = ATTACK_NEITHER_STRIKES_UNIT1_CAN_STRIKE;
		}
		else if(!unit1performsCloseCombatAttack)
		{
			result = ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE;
		}
		else
		{
			result = ATTACK_NEITHER_UNIT_CAN_STRIKE;
		}
	}
	else
	{
		cout << "error in computer" << endl;
		result = ATTACK_ERROR_FILE_IO;
	}

	return result;

	/*ADD_IF_USING_DIFFERENT_VERSION_OF_RULES_WHERE_CLOSE_COMBAT_FINISHES_IN_ONE_ROUND
	}
	*/
}






















































//preconditions: Assumes particular ModelDetails files exist and are available. Assumes ModelDetails files are fully self contained ModelDetails files - ie each ModelDetails file must contain all information including that about relevant horses etc
//
//postconditions: returns 1 if both strike, 2 if unit1 wins, 3 if unit2 wins, 4 if neither strike, 5 if neither can strike
	//							6 if unit1 wins and only Unit 1 can strike, 7 if unit1 does not strike, and only Unit 1 can strike, 8 if  3 if unit2 wins, 4 if neither strike
	//							8 if unit2 wins and only Unit 2 can strike, 9 if unit2 does not strike, and only Unit 2 can strike.
	//	10 if error

int performLongDistanceCombatBasic(char* unit1FileName, char* unit2FileName, bool unit1performsLongDistanceAttack, bool unit2performsLongDistanceAttack)
{

	//declare variables references
	Reference * initialReferenceInUnit1 = new Reference();
	Reference * initialReferenceInUnit2 = new Reference();
	Reference * topLevelReferenceInUnit1 = new Reference(true);
	Reference * topLevelReferenceInUnit2 = new Reference(true);

	//parse the scene files and build Reference linked lists
	if(!parseFile(unit1FileName, initialReferenceInUnit1, topLevelReferenceInUnit1))
	{//file does not exist
		cout << "The unit unit1FileName=" << unit1FileName << " does not exist in the directory" << endl;
		return false;
	}
	if(!parseFile(unit2FileName, initialReferenceInUnit2, topLevelReferenceInUnit2))
	{//file does not exist
		cout << "The unit unit2FileName=" << unit2FileName << " does not exist in the directory" << endl;
		return false;
	}

	performFinalUnitClassCalculations(topLevelReferenceInUnit1->subModelDetails);
	performFinalUnitClassCalculations(topLevelReferenceInUnit2->subModelDetails);
	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit1->subModelDetails))
	{
		return false;
	}
	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit2->subModelDetails))
	{
		return false;
	}
	return performLongDistanceCombat(topLevelReferenceInUnit1->subModelDetails, topLevelReferenceInUnit2->subModelDetails, unit1performsLongDistanceAttack, unit2performsLongDistanceAttack);
}


int performLongDistanceCombatNormal(char* unit1FileName, char* unit2FileName, bool unit1performsLongDistanceAttack, bool unit2performsLongDistanceAttack)
{
	//declare initial scene references
	Reference * initialReferenceInUnit1 = new Reference();
	Reference * initialReferenceInUnit2 = new Reference();
	Reference * topLevelReferenceInUnit1 = new Reference(true);
	Reference * topLevelReferenceInUnit2 = new Reference(true);

	//parse the unit files and build Reference linked lists
	if(!parseFile(unit1FileName, initialReferenceInUnit1, topLevelReferenceInUnit1))
	{//file does not exist
		cout << "The unit unit1FileName=" << unit1FileName << " does not exist in the directory" << endl;
		return false;
	}

	//parse the unit files and build Reference linked lists
	if(!parseFile(unit2FileName, initialReferenceInUnit2, topLevelReferenceInUnit2))
	{//file does not exist
		cout << "The unit unit2FileName=" << unit2FileName << " does not exist in the directory" << endl;
		return false;
	}

	//initialise unit1/unit2 references
	topLevelReferenceInUnit1->colour = int(DEFAULT_COLOUR_OF_SUB_MODEL_CURRENTLY_BEING_SEARCHED_FOR);	// this should be modified appropriatly in the future when multiple references to the same ModelDetails can be included in a scene file (of different colours)
	topLevelReferenceInUnit1->name = unit1FileName;
	topLevelReferenceInUnit2->colour = int(DEFAULT_COLOUR_OF_SUB_MODEL_CURRENTLY_BEING_SEARCHED_FOR);	// this should be modified appropriatly in the future when multiple references to the same ModelDetails can be included in a scene file (of different colours)
	topLevelReferenceInUnit2->name = unit2FileName;


	//determing the current position of Unit 1
	searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(topLevelReferenceInUnit1, initialReferenceInUnit1, NULL, true);

	performFinalUnitClassCalculations(topLevelReferenceInUnit1->subModelDetails);
	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit1->subModelDetails))
	{
		return false;
	}

	//determing the current position of Unit 2
	searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(topLevelReferenceInUnit2, initialReferenceInUnit2, NULL, true);

	performFinalUnitClassCalculations(topLevelReferenceInUnit2->subModelDetails);
	if(!performUnitOrdinatesCheck(topLevelReferenceInUnit2->subModelDetails))
	{
		return false;
	}

	//perform long distance combat
	return performLongDistanceCombat(topLevelReferenceInUnit1->subModelDetails, topLevelReferenceInUnit2->subModelDetails, unit1performsLongDistanceAttack,  unit2performsLongDistanceAttack);
}



//preconditions: units are assumed not to have moved in their previous round (ie they both are allowed to perform long distance combat in their current turn)
int performLongDistanceCombatWithSceneFile(char* unit1FileName, char* unit2FileName, int unit1ID, int unit2ID, bool unit1intendsToPerformLongDistanceAttack, bool unit2intendsToPerformLongDistanceAttack, char* thisPhaseStartSceneFileName)
{
	//declare initial scene references
	Reference * initialReferenceInThisPhaseStartScene = new Reference();
	Reference * topLevelReferenceInThisPhaseStartScene = new Reference(true);			//The information in this object is not required or meaningful, but needs to be passed into the parseFile/parseReferenceList recursive function

	//parse the scene files and build Reference linked lists
	if(!parseFile(thisPhaseStartSceneFileName, initialReferenceInThisPhaseStartScene, topLevelReferenceInThisPhaseStartScene))
	{//file does not exist
		cout << "The unit unit1FileName=" << unit1FileName << " unit1ID=" << unit1ID << " does not exist in the directory" << endl;
		return false;
	}

	return performLongDistanceCombatWithScene(unit1FileName, unit2FileName, unit1ID, unit2ID, unit1intendsToPerformLongDistanceAttack, unit2intendsToPerformLongDistanceAttack, initialReferenceInThisPhaseStartScene);

}


int performLongDistanceCombatWithScene(char* unit1FileName, char* unit2FileName, int unit1ID, int unit2ID, bool unit1intendsToPerformLongDistanceAttack, bool unit2intendsToPerformLongDistanceAttack, Reference* initialReferenceInThisPhaseStartScene)
{
	//initialise unit1/unit2 references
	Reference * unit1ReferenceInThisPhaseStartSceneFile = new Reference(unit1FileName, unit1ID, true);
	Reference * unit2ReferenceInThisPhaseStartSceneFile = new Reference(unit2FileName, unit2ID, true);

	//determing the current position of Unit 1
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit1ReferenceInThisPhaseStartSceneFile, initialReferenceInThisPhaseStartScene, NULL, false))
	{
		cout << "The unit unit1FileName=" << unit1FileName << " unit1ID=" << unit1ID << " does not exist in the given scene" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails))
	{
		return false;
	}

	//determing the current position of Unit 2
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit2ReferenceInThisPhaseStartSceneFile, initialReferenceInThisPhaseStartScene, NULL, false))
	{
		cout << "The unit unit2FileName=" << unit2FileName << " unit2ID=" << unit2ID << " does not exist in the given scene" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit2ReferenceInThisPhaseStartSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit2ReferenceInThisPhaseStartSceneFile->subModelDetails))
	{
		return false;
	}


	//calculate whether a ModelDetails can perform long distance attack based on their range and the positions between units:
	/******************************************************/
	bool unit1CanPerformLongDistanceAttack = false;
	bool unit2CanPerformLongDistanceAttack = false;

	bool unit1HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack = unit1intendsToPerformLongDistanceAttack;	//units are assumed not to have moved in their previous round
	bool unit2HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack = unit2intendsToPerformLongDistanceAttack;	//units are assumed not to have moved in their previous round


	//calculate the range of each units weapon based on the relative height between ModelDetails:
	/******************************************************/
	double longRangeDistanceBetweenTheTwoUnits;
	double unit1MaximumPrimaryRangeOfFire;
	double unit1MaximumSecondaryRangeOfFire;
	double unit1MinimumRangeOfFire;
	double unit1RangeModifier;
	double unit2MaximumPrimaryRangeOfFire;
	double unit2MaximumSecondaryRangeOfFire;
	double unit2MinimumRangeOfFire;
	double unit2RangeModifier;

	/*Method 3 - simplied - uses modified rules, assumes linear horiz/range advatange corresponding to height increase*/
	//(positionZOfUnit2-positionZOfUnit1) and not (positionZOfUnit2-positionZOfUnit1) for unit1 range modifier; as the lesser (or more negative) the z distance of a model, the higher the model is in the scene

	unit1RangeModifier = (unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y-unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y)/LDRAW_UNITS_PER_CM*RANGE_MODIFIER_BASED_ON_INCREASED_HEIGHT;
	unit2RangeModifier = (unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y-unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y)/LDRAW_UNITS_PER_CM*RANGE_MODIFIER_BASED_ON_INCREASED_HEIGHT;
	longRangeDistanceBetweenTheTwoUnits = calculateTheLongRangeAttackDistanceBetweenTwoUnits(unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.x, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.x, unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y, unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.z, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.z);

		/*
		unit1MaximumPrimaryRangeOfFire = unit1RangeModifier + unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistancePrimaryAttackBaseRange;
		unit1MaximumSecondaryRangeOfFire = unit1RangeModifier + unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceSecondaryAttackBaseRange;
		unit2MaximumPrimaryRangeOfFire = unit2RangeModifier + unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistancePrimaryAttackBaseRange;
		unit2MaximumSecondaryRangeOfFire = unit2RangeModifier + unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceSecondaryAttackBaseRange;
		*/
	unit1MinimumRangeOfFire = CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY;	//CHECK this: possibly make this setting more advanced for catapults (eg a ratio of the max range of fire distance)
	unit2MinimumRangeOfFire = CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY;	//CHECK this: possibly make this setting more advanced for catapults (eg a ratio of the max range of fire distance)
	//cout << "(currentPositionYOfUnit2-currentPositionYOfUnit1) = " << (currentPositionYOfUnit2-currentPositionYOfUnit1) << endl;
	//cout << "(currentPositionYOfUnit1-currentPositionYOfUnit2) = " << (currentPositionYOfUnit1-currentPositionYOfUnit2) << endl;

	/******************************************************/

	/*
	cout << "DEBUG: unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.x = " << unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.x << endl;
	cout << "DEBUG: unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y = " << unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y << endl;
	cout << "DEBUG: unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.z = " << unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.z << endl;
	cout << "DEBUG: unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.x = " << unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.x << endl;
	cout << "DEBUG: unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y = " << unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y << endl;
	cout << "DEBUG: unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.z = " << unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.z << endl;
	*/
	if(!unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->hasLongDistanceCombatWeapon)
	{
		cout << "Unit 1 does not have a long distance combat capable weapon" << endl;

	}
	else if(unit1HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack)
	{


		RulesClass * currentReferenceRulesClass = LRRCrulesUnitCombatDetailsAttackLongDistance;
		RecordClass * currentReferenceRecordClass = unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->recordOfUnitCombatDetailsAttackLongDistance;
		bool foundOptimumLongDistanceWeapon = false;
		while((currentReferenceRecordClass->next != NULL) && (!foundOptimumLongDistanceWeapon))
		{
			if(currentReferenceRecordClass->numberOfThisPartIDInTheUnit > 0)
			{
				int currentWeaponsRange = currentReferenceRulesClass->attribute5;
				double currentWeaponMaximumRangeOfFireWithModifier = currentWeaponsRange + unit1RangeModifier;
				if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)currentWeaponMaximumRangeOfFireWithModifier, (int)unit1MinimumRangeOfFire))
				{
					foundOptimumLongDistanceWeapon = true;

					if(currentReferenceRulesClass->name == CATAPULT_BUCKET_NAME)
					{
						unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4 * currentReferenceRecordClass->numberOfThisPartIDInTheUnit;
					}
					else
					{
						unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4;
					}
					unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackBaseRange = currentReferenceRulesClass->attribute5;
					unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->hasLongDistanceCombatWeapon = true;		//not necessary, already filled in
					unit1CanPerformLongDistanceAttack = true;
				}
			}

			currentReferenceRecordClass = currentReferenceRecordClass->next;
			currentReferenceRulesClass = currentReferenceRulesClass->next;
		}

			/*
			if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)unit1MaximumSecondaryRangeOfFire, (int)unit1MinimumRangeOfFire))
			{
				unit1CanPerformLongDistanceAttack = true;
				unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceSecondaryAttackValue;
			}
			else if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)unit1MaximumPrimaryRangeOfFire, (int)unit1MinimumRangeOfFire))
			{
				unit1CanPerformLongDistanceAttack = true;
				unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistancePrimaryAttackValue;
			}
			else
			{
				unit1CanPerformLongDistanceAttack = false;
			}
			*/
	}

	if(!unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->hasLongDistanceCombatWeapon)
	{
		cout << "unit 2 does not have a long distance combat capable weapon" << endl;
	}
	else if(unit2HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack)
	{

		RulesClass * currentReferenceRulesClass = LRRCrulesUnitCombatDetailsAttackLongDistance;
		RecordClass * currentReferenceRecordClass = unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->recordOfUnitCombatDetailsAttackLongDistance;
		bool foundOptimumLongDistanceWeapon = false;
		while((currentReferenceRecordClass->next != NULL) && (!foundOptimumLongDistanceWeapon))
		{
			if(currentReferenceRecordClass->numberOfThisPartIDInTheUnit > 0)
			{
				int currentWeaponsRange = currentReferenceRulesClass->attribute5;
				double currentWeaponMaximumRangeOfFireWithModifier = currentWeaponsRange + unit2RangeModifier;
				if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)currentWeaponMaximumRangeOfFireWithModifier, (int)unit2MinimumRangeOfFire))
				{
					foundOptimumLongDistanceWeapon = true;

					if(currentReferenceRulesClass->name == CATAPULT_BUCKET_NAME)
					{
						unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4 * currentReferenceRecordClass->numberOfThisPartIDInTheUnit;
					}
					else
					{
						unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4;
					}
					unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackBaseRange = currentReferenceRulesClass->attribute5;
					unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->hasLongDistanceCombatWeapon = true;		//not necessary, already filled in
					unit2CanPerformLongDistanceAttack = true;
				}
			}

			currentReferenceRecordClass = currentReferenceRecordClass->next;
			currentReferenceRulesClass = currentReferenceRulesClass->next;
		}

		/*
		if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)unit2MaximumSecondaryRangeOfFire, (int)unit2MinimumRangeOfFire))
		{
			unit2CanPerformLongDistanceAttack = true;
			unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceSecondaryAttackValue;
		}
		else if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)unit2MaximumPrimaryRangeOfFire, (int)unit2MinimumRangeOfFire))
		{
			unit2CanPerformLongDistanceAttack = true;
			unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistancePrimaryAttackValue;
		}
		else
		{
			unit2CanPerformLongDistanceAttack = false;
		}
		*/
	}
	/******************************************************/

	if(!unit1CanPerformLongDistanceAttack && !unit2CanPerformLongDistanceAttack)
	{
		//neither ModelDetails is within range of each other or have moved in their previous round
		return ATTACK_NEITHER_UNIT_CAN_STRIKE;
	}
	else
	{
		//perform long distance combat
		return performLongDistanceCombat(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails, unit2ReferenceInThisPhaseStartSceneFile->subModelDetails, unit1CanPerformLongDistanceAttack,  unit2CanPerformLongDistanceAttack);
	}
}





//postconditions: the validity of the movement of the units(') between scene files is not checked - this should be done in the round's movement checker routine
int performLongDistanceCombatWithConsecutiveSceneFiles(char* unit1FileName, char* unit2FileName, int unit1ID, int unit2ID, bool unit1intendsToPerformLongDistanceAttack, bool unit2intendsToPerformLongDistanceAttack, char* thisPhaseStartSceneFileName, char* preMovementPhaseSceneFileName)
{
	//declare initial scene references
	Reference * initialReferenceInThisPhaseStartScene = new Reference();
	Reference * initialReferenceInPreMovementPhaseScene = new Reference();
	Reference * topLevelReferenceInThisPhaseStartScene = new Reference(true);
	Reference * topLevelReferenceInPreMovementPhaseScene = new Reference(true);

	//parse the scene files and build Reference linked lists
	if(!parseFile(thisPhaseStartSceneFileName, initialReferenceInThisPhaseStartScene, topLevelReferenceInThisPhaseStartScene))
	{//file does not exist
		cout << "The unit unit1FileName=" << unit1FileName << " unit1ID=" << unit1ID << " does not exist in the directory" << endl;
		return false;
	}

	if(!parseFile(preMovementPhaseSceneFileName, initialReferenceInPreMovementPhaseScene, topLevelReferenceInPreMovementPhaseScene))
	{//file does not exist
		cout << "The unit unit2FileName=" << unit2FileName << " unit2ID=" << unit2ID << " does not exist in the directory" << endl;
		return false;
	}

	return performLongDistanceCombatWithConsecutiveScenes(unit1FileName, unit2FileName, unit1ID, unit2ID, unit1intendsToPerformLongDistanceAttack, unit2intendsToPerformLongDistanceAttack, initialReferenceInThisPhaseStartScene, initialReferenceInPreMovementPhaseScene);

}


int performLongDistanceCombatWithConsecutiveScenes(char* unit1FileName, char* unit2FileName, int unit1ID, int unit2ID, bool unit1intendsToPerformLongDistanceAttack, bool unit2intendsToPerformLongDistanceAttack, Reference* initialReferenceInThisPhaseStartScene, Reference* initialReferenceInPreMovementPhaseScene)
{
	//initialise unit1/unit2 references
	Reference * unit1ReferenceInThisPhaseStartSceneFile = new Reference(unit1FileName, unit1ID, true);
	Reference * unit2ReferenceInThisPhaseStartSceneFile = new Reference(unit2FileName, unit2ID, true);
	Reference * unit1ReferenceInPreMovementPhaseSceneFile = new Reference(unit1FileName, unit1ID, true);
	Reference * unit2ReferenceInPreMovementPhaseSceneFile = new Reference(unit2FileName, unit2ID, true);

	//determing the current position of Unit 1
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit1ReferenceInThisPhaseStartSceneFile, initialReferenceInThisPhaseStartScene, NULL, false))
	{
		cout << "The unit unit1FileName=" << unit1FileName << " unit1ID=" << unit1ID << " does not exist in the given scene" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit1ReferenceInThisPhaseStartSceneFile->subModelDetails))
	{
		return false;
	}

	//determing the previous  position of Unit 1
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit1ReferenceInPreMovementPhaseSceneFile, initialReferenceInPreMovementPhaseScene, NULL, false))
	{
		cout << "The unit unit1FileName=" << unit1FileName << " unit1ID=" << unit1ID << " does not exist in the given scene" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit1ReferenceInPreMovementPhaseSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit1ReferenceInPreMovementPhaseSceneFile->subModelDetails))
	{
		return false;
	}

	//determing the current position of Unit 2
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit2ReferenceInThisPhaseStartSceneFile, initialReferenceInThisPhaseStartScene, NULL, false))
	{
		cout << "The unit unit2FileName=" << unit2FileName << " unit2ID=" << unit2ID << " does not exist in the given scene" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit2ReferenceInThisPhaseStartSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit2ReferenceInThisPhaseStartSceneFile->subModelDetails))
	{
		return false;
	}

	//determing the previous position of Unit 2
	if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unit2ReferenceInPreMovementPhaseSceneFile, initialReferenceInPreMovementPhaseScene, NULL, false))
	{
		cout << "The unit unit2FileName=" << unit2FileName << " unit2ID=" << unit2ID << " does not exist in the given scene" << endl;
		return ATTACK_ERROR_FILE_IO;
	}
	performFinalUnitClassCalculations(unit2ReferenceInPreMovementPhaseSceneFile->subModelDetails);
	if(!performUnitOrdinatesCheck(unit2ReferenceInPreMovementPhaseSceneFile->subModelDetails))
	{
		return false;
	}

	//check whether chosen ModelDetails(s) can perform long distance attack during their turn based on whether or not they have moved during the round,
	/******************************************************/

	bool unit1HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack = false;
	bool unit2HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack = false;

		//26-3-06 'absolute' changed to 'relative' reasoning: /*"A movement turn does not only count for the individual " ...change rules to accomodate long distance attack if an absolute movement has occured but a relative has not for any given unit.*/
	double distanceMovedByUnit1DuringRound = calculateTheDistanceBetweenTwoPoints(unit1ReferenceInThisPhaseStartSceneFile->relativePosition.x, unit1ReferenceInPreMovementPhaseSceneFile->relativePosition.x, unit1ReferenceInThisPhaseStartSceneFile->relativePosition.y, unit1ReferenceInPreMovementPhaseSceneFile->relativePosition.y, unit1ReferenceInThisPhaseStartSceneFile->relativePosition.z, unit1ReferenceInPreMovementPhaseSceneFile->relativePosition.z);
	double distanceMovedByUnit2DuringRound = calculateTheDistanceBetweenTwoPoints(unit2ReferenceInThisPhaseStartSceneFile->relativePosition.x, unit2ReferenceInPreMovementPhaseSceneFile->relativePosition.x, unit2ReferenceInThisPhaseStartSceneFile->relativePosition.y, unit2ReferenceInPreMovementPhaseSceneFile->relativePosition.y, unit2ReferenceInThisPhaseStartSceneFile->relativePosition.z, unit2ReferenceInPreMovementPhaseSceneFile->relativePosition.z);


	if((int(distanceMovedByUnit1DuringRound) == 0) && unit1intendsToPerformLongDistanceAttack)
	{
		unit1HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack = true;
	}
	else if(unit1intendsToPerformLongDistanceAttack)
	{
		cout << "unit 1 cannot perform long distance attack as it has moved in the current round" << endl;
	}

	if((int(distanceMovedByUnit2DuringRound) == 0) && unit2intendsToPerformLongDistanceAttack)
	{
		unit2HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack = true;
	}
	else if(unit2intendsToPerformLongDistanceAttack)
	{
		cout << "unit 2 cannot perform long distance attack as it has moved in the current round" << endl;
	}


	//calculate whether a ModelDetails can perform long distance attack based on their range and the positions between units:
	/******************************************************/
	bool unit1CanPerformLongDistanceAttack = false;
	bool unit2CanPerformLongDistanceAttack = false;

	//calculate the range of each units weapon based on the relative height between ModelDetails:
	/******************************************************/
	double longRangeDistanceBetweenTheTwoUnits;
	double unit1MaximumPrimaryRangeOfFire;
	double unit1MaximumSecondaryRangeOfFire;
	double unit1MinimumRangeOfFire;
	double unit1RangeModifier;
	double unit2MaximumPrimaryRangeOfFire;
	double unit2MaximumSecondaryRangeOfFire;
	double unit2MinimumRangeOfFire;
	double unit2RangeModifier;

	/*Method 3 - simplied - uses modified rules, assumes linear horiz/range advatange corresponding to height increase*/
	//(positionZOfUnit2-positionZOfUnit1) and not (positionZOfUnit2-positionZOfUnit1) for unit1 range modifier; as the lesser (or more negative) the z distance of a model, the higher the model is in the scene

	unit1RangeModifier = (unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y-unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y)/LDRAW_UNITS_PER_CM*RANGE_MODIFIER_BASED_ON_INCREASED_HEIGHT;
	unit2RangeModifier = (unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y-unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y)/LDRAW_UNITS_PER_CM*RANGE_MODIFIER_BASED_ON_INCREASED_HEIGHT;
	longRangeDistanceBetweenTheTwoUnits = calculateTheLongRangeAttackDistanceBetweenTwoUnits(unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.x, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.x, unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y, unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.z, unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.z);

		/*
		unit1MaximumPrimaryRangeOfFire = unit1RangeModifier + unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistancePrimaryAttackBaseRange;
		unit1MaximumSecondaryRangeOfFire = unit1RangeModifier + unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceSecondaryAttackBaseRange;
		unit2MaximumPrimaryRangeOfFire = unit2RangeModifier + unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistancePrimaryAttackBaseRange;
		unit2MaximumSecondaryRangeOfFire = unit2RangeModifier + unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceSecondaryAttackBaseRange;
		*/
	unit1MinimumRangeOfFire = CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY;	//CHECK this: possibly make this setting more advanced for catapults (eg a ratio of the max range of fire distance)
	unit2MinimumRangeOfFire = CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY;	//CHECK this: possibly make this setting more advanced for catapults (eg a ratio of the max range of fire distance)
	//cout << "(currentPositionYOfUnit2-currentPositionYOfUnit1) = " << (currentPositionYOfUnit2-currentPositionYOfUnit1) << endl;
	//cout << "(currentPositionYOfUnit1-currentPositionYOfUnit2) = " << (currentPositionYOfUnit1-currentPositionYOfUnit2) << endl;

	/******************************************************/

	/*
	cout << "DEBUG: unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.x = " << unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.x << endl;
	cout << "DEBUG: unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y = " << unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.y << endl;
	cout << "DEBUG: unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.z = " << unit1ReferenceInThisPhaseStartSceneFile->absolutePosition.z << endl;
	cout << "DEBUG: unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.x = " << unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.x << endl;
	cout << "DEBUG: unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y = " << unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.y << endl;
	cout << "DEBUG: unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.z = " << unit2ReferenceInThisPhaseStartSceneFile->absolutePosition.z << endl;
	*/
	if(!unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->hasLongDistanceCombatWeapon)
	{
		cout << "Unit 1 does not have a long distance combat capable weapon" << endl;

	}
	else if(unit1HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack)
	{


		RulesClass * currentReferenceRulesClass = LRRCrulesUnitCombatDetailsAttackLongDistance;
		RecordClass * currentReferenceRecordClass = unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->recordOfUnitCombatDetailsAttackLongDistance;
		bool foundOptimumLongDistanceWeapon = false;
		while((currentReferenceRecordClass->next != NULL) && (!foundOptimumLongDistanceWeapon))
		{
			if(currentReferenceRecordClass->numberOfThisPartIDInTheUnit > 0)
			{
				int currentWeaponsRange = currentReferenceRulesClass->attribute5;
				double currentWeaponMaximumRangeOfFireWithModifier = currentWeaponsRange + unit1RangeModifier;
				if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)currentWeaponMaximumRangeOfFireWithModifier, (int)unit1MinimumRangeOfFire))
				{
					foundOptimumLongDistanceWeapon = true;

					if(currentReferenceRulesClass->name == CATAPULT_BUCKET_NAME)
					{
						unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4 * currentReferenceRecordClass->numberOfThisPartIDInTheUnit;
					}
					else
					{
						unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4;
					}
					unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackBaseRange = currentReferenceRulesClass->attribute5;
					unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->hasLongDistanceCombatWeapon = true;		//not necessary, already filled in
					unit1CanPerformLongDistanceAttack = true;
				}
			}

			currentReferenceRecordClass = currentReferenceRecordClass->next;
			currentReferenceRulesClass = currentReferenceRulesClass->next;
		}

			/*
			if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)unit1MaximumSecondaryRangeOfFire, (int)unit1MinimumRangeOfFire))
			{
				unit1CanPerformLongDistanceAttack = true;
				unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceSecondaryAttackValue;
			}
			else if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)unit1MaximumPrimaryRangeOfFire, (int)unit1MinimumRangeOfFire))
			{
				unit1CanPerformLongDistanceAttack = true;
				unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistancePrimaryAttackValue;
			}
			else
			{
				unit1CanPerformLongDistanceAttack = false;
			}
			*/
	}

	if(!unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->hasLongDistanceCombatWeapon)
	{
		cout << "unit 2 does not have a long distance combat capable weapon" << endl;
	}
	else if(unit2HasNotMovedInPreviousRoundAndIntendsToPerformLongDistanceAttack)
	{

		RulesClass * currentReferenceRulesClass = LRRCrulesUnitCombatDetailsAttackLongDistance;
		RecordClass * currentReferenceRecordClass = unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->recordOfUnitCombatDetailsAttackLongDistance;
		bool foundOptimumLongDistanceWeapon = false;
		while((currentReferenceRecordClass->next != NULL) && (!foundOptimumLongDistanceWeapon))
		{
			if(currentReferenceRecordClass->numberOfThisPartIDInTheUnit > 0)
			{
				int currentWeaponsRange = currentReferenceRulesClass->attribute5;
				double currentWeaponMaximumRangeOfFireWithModifier = currentWeaponsRange + unit2RangeModifier;
				if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)currentWeaponMaximumRangeOfFireWithModifier, (int)unit2MinimumRangeOfFire))
				{
					foundOptimumLongDistanceWeapon = true;

					if(currentReferenceRulesClass->name == CATAPULT_BUCKET_NAME)
					{
						unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4 * currentReferenceRecordClass->numberOfThisPartIDInTheUnit;
					}
					else
					{
						unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4;
					}
					unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackBaseRange = currentReferenceRulesClass->attribute5;
					unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->hasLongDistanceCombatWeapon = true;		//not necessary, already filled in
					unit2CanPerformLongDistanceAttack = true;
				}
			}

			currentReferenceRecordClass = currentReferenceRecordClass->next;
			currentReferenceRulesClass = currentReferenceRulesClass->next;
		}

		/*
		if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)unit2MaximumSecondaryRangeOfFire, (int)unit2MinimumRangeOfFire))
		{
			unit2CanPerformLongDistanceAttack = true;
			unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceSecondaryAttackValue;
		}
		else if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(longRangeDistanceBetweenTheTwoUnits, (int)unit2MaximumPrimaryRangeOfFire, (int)unit2MinimumRangeOfFire))
		{
			unit2CanPerformLongDistanceAttack = true;
			unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackValue = unit2ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistancePrimaryAttackValue;
		}
		else
		{
			unit2CanPerformLongDistanceAttack = false;
		}
		*/
	}
	/******************************************************/

	if(!unit1CanPerformLongDistanceAttack && !unit2CanPerformLongDistanceAttack)
	{
		//neither ModelDetails is within range of each other or have moved in their previous round
		return ATTACK_NEITHER_UNIT_CAN_STRIKE;
	}
	else
	{
		//perform long distance combat
		return performLongDistanceCombat(unit1ReferenceInPreMovementPhaseSceneFile->subModelDetails, unit2ReferenceInPreMovementPhaseSceneFile->subModelDetails, unit1CanPerformLongDistanceAttack,  unit2CanPerformLongDistanceAttack);
	}
}



//preconditions: Assumes particular ModelDetails objects (unit1 and unit2) are fully self contained - ie they contain all information including that about horses etc
//		(even though a knights respective horse may have been found in a higher level file during initial parse)
//
//postconditions: returns 1 if both strike, 2 if unit1 wins, 3 if unit2 wins, 4 if neither strike, 5 if neither can strike
	//							6 if unit1 wins and only Unit 1 can strike, 7 if unit1 does not strike, and only Unit 1 can strike, 8 if  3 if unit2 wins, 4 if neither strike
	//							8 if unit2 wins and only Unit 2 can strike, 9 if unit2 does not strike, and only Unit 2 can strike.
	//	10 if error
int performLongDistanceCombat(ModelDetails * unit1, ModelDetails * unit2, bool unit1performsLongDistanceAttack, bool unit2performsLongDistanceAttack)
{
	//cout << "\n\nDEBUG: unit1FileName = " << unit1FileName << endl;
	//cout << "\n\nDEBUG: unit2FileName = " << unit2FileName << endl;
	int result;

	int unit1Attack = unit1->longDistanceAttackValue;
	int unit1Defence = unit1->defenceTotal;
	int unit2Attack = unit2->longDistanceAttackValue;
	int unit2Defence = unit2->defenceTotal;

#ifdef GAME_OUTPUT_COMBAT_ENTRY_STATISTICS
	cout << "\nUnit 1 Statistics:\n" << endl;
	cout << "Defence Level = " << invertLevel(unit1->defenceTotal) << endl;
	cout << "Long Distance Attack Level = " << invertLevel(unit1->longDistanceAttackValue) << endl;
	cout << "\nUnit 2 Statistics:\n" << endl;
	cout << "Defence Level = " << invertLevel(unit2->defenceTotal) << endl;
	cout << "Long Distance Attack Level = " << invertLevel(unit2->longDistanceAttackValue) << endl;
#endif

	bool unit1Strikes = false;
	bool unit2Strikes = false;

	//added 18-3-06 check if units have long distance capable weapons

	if(unit1performsLongDistanceAttack)
	{
		if(!unit1->hasLongDistanceCombatWeapon)
		{
			cout << "Error, unit 1 does not have a long distance combat capable weapon" << endl;
			result = ATTACK_NEITHER_UNIT_CAN_STRIKE;
		}
		else
		{
			int diceThrown1 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit1Attack)*DICE_MODIFIER+1.0);	 //+1 for 0.0001->11.9999 ... 1.0001->12.9999 ... 1->12 (round down)
			int diceThrown2 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit2Defence)*DICE_MODIFIER+1.0);

			cout << "unit 1 throws: " << diceThrown1 << " in attack using d" << (unit1Attack)*DICE_MODIFIER <<endl;
			cout << "unit 2 throws: " << diceThrown2 << " in defence using d" << (unit2Defence)*DICE_MODIFIER << endl;

			if(diceThrown1 > diceThrown2)
			{
				unit1Strikes = true;
			}
		}
	}

	//added 18-3-06 check if units have long distance capable weapons

	if(unit2performsLongDistanceAttack)
	{
		if(!unit2->hasLongDistanceCombatWeapon)
		{
			cout << "Error, unit 2 does not have a long distance combat capable weapon" << endl;
			result = ATTACK_NEITHER_UNIT_CAN_STRIKE;
		}
		else
		{
			int diceThrown3 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit2Attack)*DICE_MODIFIER+1.0);
			int diceThrown4 = int(((double(abs((short)rand())))/(ABS_OF_SHORT))*((double)unit1Defence)*DICE_MODIFIER+1.0);
			cout << "unit 2 throws: " << diceThrown3 << " in attack using d" << (unit2Attack)*DICE_MODIFIER <<endl;
			cout << "unit 1 throws: " << diceThrown4 << " in defence using d" << (unit1Defence)*DICE_MODIFIER << endl;

			if(diceThrown3 > diceThrown4)
			{
				unit2Strikes = true;
			}
		}
	}

	if(result != ATTACK_NEITHER_UNIT_CAN_STRIKE)
	{
		if(unit1Strikes && unit2Strikes)
		{
			//nothing happens, no blows were fatal
			cout << "Both units strike. No strikes were fatal. [No units should be removed from scene file]." << endl;
			return ATTACK_BOTH_UNITS_STRIKE_BOTH_CAN_STRIKE;
		}
		else if(unit1Strikes && !unit2Strikes)
		{
			cout << "Unit 1 wins. [Unit 2 should be removed from the scene file]." << endl;
			if(unit2performsLongDistanceAttack)
			{
				result = ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE;
			}
			else
			{
				result = ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE;
			}
		}
		else if(!unit1Strikes && unit2Strikes)
		{
			cout << "Unit 2 wins. [Unit 1 should be removed from the scene file]." << endl;
			if(unit1performsLongDistanceAttack)
			{
				result = ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE;
			}
			else
			{
				result = ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE;
			}

		}
		else if(!unit1Strikes && !unit2Strikes)
		{
			cout << "The unit(s) intending to strike did not. [No units should be removed from scene file]." << endl;
			if(unit1performsLongDistanceAttack && unit2performsLongDistanceAttack)
			{
				result = ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE;
			}
			else if(!unit2performsLongDistanceAttack)
			{
				result = ATTACK_NEITHER_STRIKES_UNIT1_CAN_STRIKE;
			}
			else if(!unit1performsLongDistanceAttack)
			{
				result = ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE;
			}
			else
			{
				result = ATTACK_NEITHER_UNIT_CAN_STRIKE;
			}
		}
		else
		{
			cout << "error in computer" << endl;
			result = ATTACK_ERROR_FILE_IO;
		}
	}

	return result;
}











































//combat unit properties post parser (reference list searching)

/****************************************************************
/
/ searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel
/
/****************************************************************/

bool searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(Reference * referenceBeingSearchedFor, Reference * initialReference, Reference * parentReference, bool isFillingParentUnitWithAllCombatRelevantChildModelDetails)
{
	bool particularSubModelFoundDuringSearch = false;
	Reference * currentReference = initialReference;

	//cout << "DEBUG: c1" << endl;

	while(currentReference->next != NULL)
	{

		//cout << "\nDEBUG: c2" << endl;
		//cout << "currentReference->name = " << currentReference->name << endl;
		//cout << "referenceBeingSearchedFor->name = " << referenceBeingSearchedFor->name << endl;

		if(currentReference->isSubModelReference)
		{
			bool currentReferenceFoundIsTheParticularSubmodelBeingLookedFor = false;


			if(isFillingParentUnitWithAllCombatRelevantChildModelDetails)
			{
				addAllCombatRelevantChildModelDetailsIntoAParentUnit(currentReference->subModelDetails, referenceBeingSearchedFor->subModelDetails);
				//cout << "\nDEBUG: c4" << endl;
				//cout << "currentReference->name = " << currentReference->name << endl;
				//cout << "referenceBeingSearchedFor->name = " << referenceBeingSearchedFor->name << endl;

			}

			if(!isFillingParentUnitWithAllCombatRelevantChildModelDetails)
			{
				if(compareSubmodelNamesAndIfSameCopySubmodelReference(referenceBeingSearchedFor, currentReference))
				{
					particularSubModelFoundDuringSearch = true;
					currentReferenceFoundIsTheParticularSubmodelBeingLookedFor = true;
				}
			}

			if(searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(referenceBeingSearchedFor, currentReference->firstReferenceWithinSubModel, currentReference, (currentReferenceFoundIsTheParticularSubmodelBeingLookedFor|isFillingParentUnitWithAllCombatRelevantChildModelDetails)))
			{
				if(!isFillingParentUnitWithAllCombatRelevantChildModelDetails) //always TRUE	//(!isFillingParentUnitWithAllCombatRelevantChildModelDetails) is implied true as searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel has returned true.
				{
					particularSubModelFoundDuringSearch = true;

					//no longer used due to algorithm break down
					//copyCombatRelevantInheritedImmediateParentUnitDetailsIntoChildObject(referenceBeingSearchedFor->subModelDetails, parentReference->subModelDetails);
				}
			}
		}

		//cout << "DEBUG: c6 currentReference->next = " << currentReference->next << endl;
		currentReference = currentReference->next;
		//cout << "DEBUG: c7 currentReference->next = " << currentReference->next << endl;
	}

	return particularSubModelFoundDuringSearch;
}

bool compareSubmodelNamesAndIfSameCopySubmodelReference(Reference * referenceBeingSearchedFor, Reference * referenceCurrentlyFoundDuringSearch)
{
	bool particularSubModelFoundDuringSearch = false;
	//cout << "DEBUG: d1" << endl;
	//cout << "DEBUG: d1 referenceBeingSearchedFor->name =" << referenceBeingSearchedFor->name << endl;
	//cout << "DEBUG: d1 referenceCurrentlyFoundDuringSearch->name =" << referenceCurrentlyFoundDuringSearch->name << endl;
	if(referenceBeingSearchedFor->name == referenceCurrentlyFoundDuringSearch->name)
	{
		//cout << "DEBUG: d2 referenceBeingSearchedFor->colour =" << referenceBeingSearchedFor->colour << endl;
		//cout << "DEBUG: d2 referenceCurrentlyFoundDuringSearch->colour =" << referenceCurrentlyFoundDuringSearch->colour << endl;
		if(referenceBeingSearchedFor->colour == referenceCurrentlyFoundDuringSearch->colour)
		{
			copyReferencePosition(referenceBeingSearchedFor, referenceCurrentlyFoundDuringSearch);

			copyAllUnitDetails(referenceBeingSearchedFor->subModelDetails, referenceCurrentlyFoundDuringSearch->subModelDetails);

			particularSubModelFoundDuringSearch = true;
			//cout << "DEBUG: d5" << endl;
		}
		else
		{
			//do not do anything with this particular submodel/part/ModelDetails Reference, it is the wrong one (it is not of its stated colour)
		}
	}
	else
	{
		//do not do anything with this particular submodel/part/ModelDetails Reference, it is the wrong one (it is not of the required)
	}
	return particularSubModelFoundDuringSearch;
}



































//combat calculations



bool calculateIfTwoUnitsAreWithinAGivenLongRangeAttackDistance(double positionXOfUnit1, double positionXOfUnit2, double positionYOfUnit1, double positionYOfUnit2, double positionZOfUnit1, double positionZOfUnit2, int maximumDistance, int minimumDistance)
{
	bool result;

	distanceBetweenTheTwoUnits = calculateTheLongRangeAttackDistanceBetweenTwoUnits(positionXOfUnit1, positionXOfUnit2, positionYOfUnit1, positionYOfUnit2, positionZOfUnit1, positionZOfUnit2);

	result = calculateIfUnitIsWithinAGivenLongRangeAttackDistance(distanceBetweenTheTwoUnits, maximumDistance, minimumDistance);

	return result;
}

bool calculateIfUnitIsWithinAGivenLongRangeAttackDistance(double distanceBetweenTheTwoUnits, int maximumDistance, int minimumDistance)
{
	bool result;

	if(((int)distanceBetweenTheTwoUnits <= maximumDistance) && ((int)distanceBetweenTheTwoUnits >= minimumDistance))
	{
		//cout << "\nDEBUG: Units are within Range for long range attack. Distance between units = " << distanceBetweenTheTwoUnits << endl;
		//cout << "Maximum Distance = " << maximumDistance << endl;
		result = true;
	}
	else
	{
		cout << "\nA ModelDetails is out of Range for long range attack. Distance between units = " << distanceBetweenTheTwoUnits << endl;
		cout << "Maximum Firing Distance of Unit = " << maximumDistance << ", Minimum Firing Distance of Unit = " << minimumDistance << endl;
		/*
		cout << "positionXOfUnit1 = " << positionXOfUnit1 << ", positionXOfUnit2 = " << positionXOfUnit2 << endl;
		cout << "positionYOfUnit1 = " << positionYOfUnit1 << ", positionYOfUnit2 = " << positionYOfUnit2 << endl;
		cout << "positionZOfUnit1 = " << positionZOfUnit1 << ", positionZOfUnit2 = " << positionZOfUnit2 << endl;
		*/
		result = false;
	}

	return result;
}


double calculateTheLongRangeAttackDistanceBetweenTwoUnits(vec * positionOfUnit1, vec * positionOfUnit2)
{
	vec positionOfUnit1ConvertedToLdraw;
	vec positionOfUnit2ConvertedToLdraw;
	copyVectors(positionOfUnit1ConvertedToLdraw, positionOfUnit1);
	copyVectors(positionOfUnit2ConvertedToLdraw, positionOfUnit2);
	multipleVectorByScalar(positionOfUnit1ConvertedToLdraw, LDRAW_UNITS_PER_CM);
	multipleVectorByScalar(positionOfUnit2ConvertedToLdraw, LDRAW_UNITS_PER_CM);

	return calculateTheDistanceBetweenTwoPoints(positionOfUnit1ConvertedToLdraw, positionOfUnit2ConvertedToLdraw);
}

double calculateTheLongRangeAttackDistanceBetweenTwoUnits(double positionXOfUnit1, double positionXOfUnit2, double positionYOfUnit1, double positionYOfUnit2, double positionZOfUnit1, double positionZOfUnit2)
{

#ifndef USE_OLD_LONG_RANGE_ATTACK_DISTANCE_CALC
	return calculateTheDistanceBetweenTwoPoints(positionXOfUnit1/LDRAW_UNITS_PER_CM, positionXOfUnit2/LDRAW_UNITS_PER_CM, positionYOfUnit1/LDRAW_UNITS_PER_CM, positionYOfUnit2/LDRAW_UNITS_PER_CM, positionZOfUnit1/LDRAW_UNITS_PER_CM, positionZOfUnit2/LDRAW_UNITS_PER_CM);
#else
	double xDistanceBetweenTheTwoUnits = fabs((positionXOfUnit1/LDRAW_UNITS_PER_CM) -  (positionXOfUnit2/LDRAW_UNITS_PER_CM));
	double zDistanceBetweenTheTwoUnits = fabs((positionZOfUnit1/LDRAW_UNITS_PER_CM) -  (positionZOfUnit2/LDRAW_UNITS_PER_CM));
	double distanceBetweenTheTwoUnits = sqrt(pow(xDistanceBetweenTheTwoUnits, 2) + pow(zDistanceBetweenTheTwoUnits, 2));
	return distanceBetweenTheTwoUnits;
#endif

}



//This function is not currently used (instead simplified calculations of range advantage are calculated based on original Rules
double calculateExtraHorizontalDistanceOfProjectileWithHeightAdvantage(double verticalHeightAdvantage, double maxHorizRangeOfWeapon)
{
	return false;
}






















