/*******************************************************************************
 *
 * File Name: LRRCplayerClass.cpp
 * Author: Richard Baxter
 * Project: Lego Rules CG Rounds Checker
 * Project Version: 3e2a 16-Feb-07
 * Project First Internal Release: 1aXx 18-Sept-05 (C)
 * Project Second Internal Release: 2aXx 02-April-06 (convert to C++)
 * Project Third Internal Release: 2b7d 26-Sept-06 (added sprites)
 * Project Fourth Internal Release: 2c4a 25-Feb-07 [rotation of models supported, copyCombatRelevantInheritedImmediateParentUnitDetailsIntoChildObject() no longer used]
 * Project Fifth Internal Release: 2d11a 3-June-07
 *
 *******************************************************************************/



/*LDreferenceClass.cpp: Defines a class for references to submodels/parts/units within a scene file*/

#include "LRRCplayerClass.h"
#include "XMLrulesClass.h"

static long PLAYER_BANK_ACCOUNT_INITIAL_DEFAULT;
static long PLAYER_ROUND_RITUAL_COMBAT_POINTS_ALLOCATED;
static long PLAYER_ROUND_RITUAL_BUILDING_POINTS_ALLOCATED;
static double PLAYER_MAXIMUM_BUILD_DISTANCE_FROM_STARTING_POST;
static int BUILDING_DEFAULT_MOD;


void fillInPlayerClassExternVariables()
{
	RulesClass * currentReferenceRulesClass = LRRCrulesMiscellaneous;

	while(currentReferenceRulesClass->next != NULL)
	{
		if(currentReferenceRulesClass->name == PLAYER_BANK_ACCOUNT_INITIAL_DEFAULT_NAME)
		{
			PLAYER_BANK_ACCOUNT_INITIAL_DEFAULT = long(currentReferenceRulesClass->fractionalValue);
		}
		else if(currentReferenceRulesClass->name == PLAYER_ROUND_RITUAL_COMBAT_POINTS_ALLOCATED_NAME)
		{
			PLAYER_ROUND_RITUAL_COMBAT_POINTS_ALLOCATED = long(currentReferenceRulesClass->fractionalValue);
		}
		else if(currentReferenceRulesClass->name == PLAYER_ROUND_RITUAL_BUILDING_POINTS_ALLOCATED_NAME)
		{
			PLAYER_ROUND_RITUAL_BUILDING_POINTS_ALLOCATED = long(currentReferenceRulesClass->fractionalValue);
		}
		else if(currentReferenceRulesClass->name == PLAYER_MAXIMUM_BUILD_DISTANCE_FROM_STARTING_POST_NAME)
		{
			PLAYER_MAXIMUM_BUILD_DISTANCE_FROM_STARTING_POST = currentReferenceRulesClass->fractionalValue;
		}
		else if(currentReferenceRulesClass->name == BUILDING_DEFAULT_MOD_NAME)
		{
			BUILDING_DEFAULT_MOD = int(currentReferenceRulesClass->fractionalValue);
		}
		else
		{

		}

		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}
}



Player::Player(void)
{
	//Define default values for Player

	name = "";
	id = 0;			//player ID - Eg colour
	credits = PLAYER_BANK_ACCOUNT_INITIAL_DEFAULT;
	startPosition.x = 0.0;
	startPosition.y = 0.0;
	startPosition.z = 0.0;
	currentPhase = GAME_PHASE_GENERIC;
	playerIsAI = false;
	status = true;

#ifdef USE_ANN
	creativity = 0.2;		//a small degree of creativity is assigned to player for now...

	firstExperience = new Experience();
	currentExperience = firstExperience;

	firstUnitInUnitList = NULL;
	currentObjectInUnitList = NULL;

	firstInputNeuronInNetwork = NULL;
	firstOutputNeuronInNetwork = NULL;
	numberOfInputNeurons = 0;
	numberOfOutputNeurons = 0;
#endif

	next = NULL;
}

/*
Player::Player(string playerName, int playerID, int playerInitialCredits)
{
	//Define initial values for Player

	name = playerName;
	id = playerID;			//player ID - Eg colour
	credits = playerInitialCredits;
	startPosition.x = 0.0;
	startPosition.y = 0.0;
	startPosition.z = 0.0;
	currentPhase = GAME_PHASE_GENERIC;
	playerIsAI = false;

#ifdef USE_ANN
	firstExperience = new Experience();
	currentExperience = firstExperience;

	firstUnitInUnitList = NULL;

#endif

	next = NULL;
}

Player::Player(string playerName, int playerID, int playerInitialCredits, vec* playerStartPosition)
{
	//Define initial values for Player

	name = playerName;
	id = playerID;			//player ID - Eg colour
	credits = playerInitialCredits;
	copyVectors(&startPosition, playerStartPosition);
	currentPhase = GAME_PHASE_GENERIC;
	playerIsAI = false;

#ifdef USE_ANN
	firstExperience = new Experience();
	currentExperience = firstExperience;

	firstUnitInUnitList = NULL;

#endif

	next = NULL;
}
*/


Player::~Player()
{
    //insert explict reference to delete any memory structures allocated automatically during construction (there are currently none);
}

void fillPlayerDetails(Player * p, string playerName, int playerID, int playerInitialCredits)
{
	p->name = playerName;
	p->id = playerID;			//player ID - Eg colour
	p->credits = playerInitialCredits;
}

void fillPlayerDetails(Player * p, string playerName, int playerID, int playerInitialCredits, vec* playerStartPosition)
{
	p->name = playerName;
	p->id = playerID;			//player ID - Eg colour
	p->credits = playerInitialCredits;
	copyVectors(&(p->startPosition), playerStartPosition);
}


Player * findPlayer(Player * initialPlayerInList, int playerID)
{
	Player * playerThatWasLocated = NULL;

	Player * currentPlayerInList = initialPlayerInList;
	while(currentPlayerInList->next != NULL)
	{
		if(currentPlayerInList->id == playerID)
		{
			playerThatWasLocated = currentPlayerInList;
		}
		currentPlayerInList = currentPlayerInList->next;
	}

	return playerThatWasLocated;
}


